/*
+-------------------------------- DISCLAIMER ---------------------------------+
|                                                                             |
| This application program is provided to you free of charge as an example.   |
| Despite the considerable efforts of Euresys personnel to create a usable    |
| example, you should not assume that this program is error-free or suitable  |
| for any purpose whatsoever.                                                 |
|                                                                             |
| EURESYS does not give any representation, warranty or undertaking that this |
| program is free of any defect or error or suitable for any purpose. EURESYS |
| shall not be liable, in contract, in torts or otherwise, for any damages,   |
| loss, costs, expenses or other claims for compensation, including those     |
| asserted by third parties, arising out of or in connection with the use of  |
| this program.                                                               |
|                                                                             |
+-----------------------------------------------------------------------------+
*/

//
//  FirstExpert Project
//
//  FirstExpertDoc class definition
//  File: FirstExpertDoc.cpp
//////////////////////////////////////////////////////////////////////////////////


// Application Header
//////////////////////////////////////////////////////////////////////////////////
#include "FirstExpert.h"

// Global functions and variables
//////////////////////////////////////////////////////////////////////////////////

// + MultiCam FirstExpert sample program
const wxEventType EG_EVT_REFRESH = wxNewEventType();

void ErrorDisplay(const char *Msg, int Code)
{
    wxString str = wxString::FromAscii(Msg);
    str += wxT("\nReturn value = ");
    str << Code;
    wxMessageDialog dlg(NULL,str,wxT("First Expert"), wxICON_ERROR);
    dlg.ShowModal();
}

void GlobalCallback(PMCCALLBACKINFO CallBackInfo)
{
    FirstExpertDoc *pDoc = (FirstExpertDoc*)CallBackInfo->Context;
    pDoc->MultiCam_CallBackFunction(CallBackInfo);
}

// Macros
//////////////////////////////////////////////////////////////////////////////////
IMPLEMENT_DYNAMIC_CLASS(FirstExpertDoc, wxDocument)

// Event Table
//////////////////////////////////////////////////////////////////////////////////
BEGIN_EVENT_TABLE(FirstExpertDoc, wxDocument)
EVT_MENU(ID_GRAB, FirstExpertDoc::OnGrab)
END_EVENT_TABLE()

// Constructors 
//////////////////////////////////////////////////////////////////////////////////
FirstExpertDoc::FirstExpertDoc()
{
    m_hChannel = 0;
    m_hSurface = 0;
    m_n32ImageWidth = 0;
    m_n32ImageHeight = 0;

    m_pAllocatedBuffer=NULL;

    wxString text;
    text.Printf(wxT("This program will use the following Setting :\n\nCamera File   (.CAM) : \"%s\"")
        , wxT(EURESYS_SAMPLE_PROGRAM_CAM_FILENAME));

    wxMessageDialog dlg(NULL,text,wxT("FirstExpert"), wxICON_INFORMATION);
    dlg.ShowModal();
    text.Printf(wxT("Please make sure you have a\n%s\ncamera connected in medium configuration"), wxT(EURESYS_SAMPLE_CAMERA_MODEL));
    wxMessageDialog dlg2(NULL,text,wxT("FirstExpert"), wxICON_INFORMATION);
    dlg2.ShowModal();
}

FirstExpertDoc::~FirstExpertDoc()
{
    MultiCam_Exit();
}

// Event Handlers 
//////////////////////////////////////////////////////////////////////////////////
bool FirstExpertDoc::OnNewDocument()
{
    if (MultiCam_Init())
    {
        ((wxGetApp().GetMainFrame())->GetCanvas())->InitScrollbars(m_n32ImageWidth, m_n32ImageHeight);

        return true;
    }
    else 
    {
        wxMessageDialog dlg(NULL,wxT("Error: Cannot initialize MultiCam application."), wxT("First Expert"), wxICON_ERROR);
        dlg.ShowModal();

        return false;
    }
}

bool FirstExpertDoc::MultiCam_Init()
{
    MCSTATUS status = MC_OK;
//    INT32 n32ModifiedPointer;
    INT64 n32ModifiedPointer;

    char pText[128];

    // Establish communication with the driver
    status = McOpenDriver(NULL);
    if (status != MC_OK)
    {
        ErrorDisplay("Error: McOpenDriver failed.", status);
        return false ;
    }

    // Create a channel
    status = McCreateNm("CHANNEL", &m_hChannel);
    if (status != MC_OK)
    {
        sprintf(pText, "Error: McCreateNm Channel failed.");
        ErrorDisplay(pText, status);
        return false;
    }

    // Associate a board with the channel 
    status = McSetParamInt(m_hChannel, MC_DriverIndex, 0);
    if (status != MC_OK)
    {
        ErrorDisplay("Error: McSetParamInt MC_DriverIndex failed.", status);
        return false;
    }

    // Associate a connector with the channel 
    status = McSetParamInt(m_hChannel, MC_Connector, MC_Connector_M);
    if (status != MC_OK)
    {
        ErrorDisplay("Error: McSetParamInt MC_Connector failed.", status);
        return false;
    }

    // Associate a camera file to the channel
    status = McSetParamStr(m_hChannel, MC_CamFile, EURESYS_SAMPLE_PROGRAM_CAM_FILENAME);
    if (status != MC_OK)
    {
        sprintf(pText, "Error: McSetParamStr MC_CamFile %s failed.", EURESYS_SAMPLE_PROGRAM_CAM_FILENAME);
        ErrorDisplay(pText, status);
        return false;
    }

    // Register callback
    status = McRegisterCallback(m_hChannel, GlobalCallback, this);
    if (status != MC_OK)
    {
        ErrorDisplay("Error: McRegisterCallback failed.", status);
        return false;
    }

    // Enable event
    //  status = McSetParamInt(m_hChannel, MC_SignalEnable + MC_SIG_SURFACE_FILLED, MC_SignalEnable_ON);
    status = McSetParamInt(m_hChannel, MC_SignalEnable + MC_SIG_SURFACE_PROCESSING, MC_SignalEnable_ON);
    if (status != MC_OK)
    {
        ErrorDisplay("Error: McSetParamInt MC_SignalEnable failed.", status);
        return false;
    }

    // Retrieve image parameters
    status = McGetParamInt(m_hChannel, MC_ImageSizeY, &m_n32ImageHeight);
    if (status != MC_OK)
    {
        ErrorDisplay("Error: McGetParamInt MC_ImageSizeY failed.", status);
        return false;
    }

    status = McGetParamInt(m_hChannel, MC_ImageSizeX, &m_n32ImageWidth);
    if (status != MC_OK)
    {
        ErrorDisplay("Error: McGetParamInt MC_ImageSizeX failed.", status);
        return false;
    }

    // Check the color format
    INT32 n32ColorFormat;
    status = McGetParamInt(m_hChannel, MC_ColorFormat, &n32ColorFormat);
    if (status != MC_OK)
    {
        ErrorDisplay("Error: McGetParamInt MC_ColorFormat failed.", status);
        return false;
    }

    if (n32ColorFormat!=MC_ColorFormat_Y8)
    {
        wxMessageDialog dlg(NULL,wxT("This sample program was designed to operate a MultiCam compliant frame-grabber\n"
            "and perform image acquisitions with the Y8 color format.\n\n"
            "You selected to operate a mode with another color format.\n\n"
            "Please use another sample program supporting your color format."),
            wxT("FirstExpert"),
            wxICON_INFORMATION);
        dlg.ShowModal();
        return false ;
    }

    // Create an image buffer

    // Get buffer size
    INT32 n32ImageSize;

    status = McGetParamInt(m_hChannel, MC_BufferSize, &n32ImageSize);
    if (status != MC_OK) 
    {
        ErrorDisplay("Error: McGetParamInt MC_BufferSize failed.", status);
        return false;
    }

    // Get image pitch
    INT32 n32ImagePitch;
    status = McGetParamInt(m_hChannel, MC_BufferPitch, &n32ImagePitch);
    if (status != MC_OK) 
    {
        ErrorDisplay("Error: McGetParamInt MC_BufferPitch failed.", status);
        return false;
    }

    // Pointer manipulation needed to align the MC_SurfaceAddr pointer to pages of memory 
    m_pAllocatedBuffer = malloc(n32ImageSize+0xF);
    n32ModifiedPointer=(INT64)m_pAllocatedBuffer;
    n32ModifiedPointer+=(-n32ModifiedPointer)&0xF;
    m_pImageBuffer=(PUINT8)n32ModifiedPointer;

    if (m_pAllocatedBuffer == NULL) 
    {
        ErrorDisplay("Error: Cannot allocate image buffer.", 0);
        return false;
    }

    // Create a surface
    status = McCreate(MC_DEFAULT_SURFACE_HANDLE, &m_hSurface);
    if (status != MC_OK) 
    {
        ErrorDisplay("Error: McCreate Surface failed.", status);
        return false;
    }

    // Set surface parameters
    status = McSetParamPtr(m_hSurface, MC_SurfaceSize,(PVOID) n32ImageSize);
    if (status != MC_OK) 
    {
        ErrorDisplay("Error: Cannot set surface size.", status);
        return false;
    }

    status = McSetParamPtr(m_hSurface, MC_SurfacePitch, (PVOID) n32ImagePitch);
    if (status != MC_OK) 
    {
        ErrorDisplay("Error: Cannot set surface pitch.", status);
        return false;
    }

    status = McSetParamPtr(m_hSurface, MC_SurfaceAddr, (PVOID) n32ModifiedPointer);
    if (status != MC_OK) 
    {
        ErrorDisplay("Error: Cannot set surface buffer.", status);
        return false;
    }

    // Surface Context Information
    status = McSetParamPtr(m_hSurface, MC_SurfaceContext, (PVOID) n32ModifiedPointer);
    if (status != MC_OK) 
    {
        ErrorDisplay("Error: Cannot set surface context information.", status);
        return false;
    }

    // Associate surface with channel
    status = McSetParamInst(m_hChannel, MC_Cluster, m_hSurface);
    if (status != MC_OK) 
    {
        ErrorDisplay("Error: Cannot add surface to cluster", status);
        return false;
    }

    // Set Channel to Idle state
    status = McSetParamInt(m_hChannel, MC_ChannelState, MC_ChannelState_IDLE); 
    if (status != MC_OK)
    {
        ErrorDisplay("Error: Cannot put channel in IDLE state.", status);
        return false;
    }

    wxGetApp().SetInitialized();

    return true;
}

void FirstExpertDoc::MultiCam_Exit()
{
    MCSTATUS status;

    // Delete Channel Instance
    if (m_hChannel)
    {
        status = McDelete(m_hChannel);
        if (status != MC_OK)
            ErrorDisplay("Error: McDelete m_ChannelInstance failed.", status);
    }

    // Delete Surface Instance
    if (m_hSurface)
    {
        status = McDelete(m_hSurface);
        if (status != MC_OK)
            ErrorDisplay("Error: McDelete m_SurfaceInstance failed.", status);
    }

    // Free allocated memory
    if (m_pAllocatedBuffer) 
        free(m_pAllocatedBuffer);

    // Terminate communication with the driver
    status = McCloseDriver();
    if (status != MC_OK)
        ErrorDisplay("Error: McCloseDriver failed.", status);
}

void FirstExpertDoc::MultiCam_CallBackFunction(PMCCALLBACKINFO CInfo)
{
    MCSTATUS status = MC_OK;
    MCHANDLE m_hSurface;
    INT64 image_address;
    INT64 image_size;
    INT64 context;

    switch(CInfo->Signal)
    {
    case MC_SIG_SURFACE_PROCESSING://MC_SIG_SURFACE_FILLED:
        {
            m_hSurface = CInfo->SignalInfo;

            // Retrieving miscellaneous informations about our image
            status = McGetParamPtr(m_hSurface, MC_SurfaceAddr, (PVOID*)&image_address);
            if (status != MC_OK) break;

            status = McGetParamPtr(m_hSurface, MC_SurfaceSize, (PVOID*) &image_size);
            if (status != MC_OK) break;

            status = McGetParamPtr(m_hSurface, MC_SurfaceContext,(PVOID*)&context);
            if (status != MC_OK) break;

            wxCommandEvent eventRefresh(EG_EVT_REFRESH);
            ::wxPostEvent(wxGetApp().GetMainFrame(), eventRefresh);

            break;
        }
    default:
        break;
    }
}

void FirstExpertDoc::OnGrab(wxCommandEvent& event)
{
    MCSTATUS status = MC_OK;

    // Set Channel to Idle state
    status = McSetParamInt(m_hChannel, MC_ChannelState, MC_ChannelState_IDLE);
    if (status != MC_OK)
        ErrorDisplay("Error: Cannot set channel to IDLE state.", status);

    // Set AcquisitionMode to "PAGE"
    status = McSetParamInt(m_hChannel, MC_AcquisitionMode, MC_AcquisitionMode_PAGE);
    if (status != MC_OK)
        ErrorDisplay("Error: Cannot set aquisition mode to PAGE.", status);

    // Set SeqLength_Pg to "1" -> one single acquisition (one page)
    status = McSetParamInt(m_hChannel, MC_SeqLength_Pg, 1);
    if (status != MC_OK)
        ErrorDisplay("Error: Cannot set SeqLength_Pg to 1.", status);

    // Activating acquisition sequence for the channel
    status = McSetParamInt(m_hChannel, MC_ChannelState, MC_ChannelState_ACTIVE);
    if (status != MC_OK)
        ErrorDisplay("Error: Cannot set channel to ACTIVE state.", status);

    // Generating a software trigger to start the acquisition
    status = McSetParamInt(m_hChannel, MC_ForceTrig, MC_ForceTrig_TRIG);
    if (status != MC_OK)
        ErrorDisplay("Error: Cannot generate software trigger.", status);
}

// - MultiCam FirstExpert sample program
